/*
 * (C) Copyright Christopher Diggins 2005
 * (C) Copyright Pablo Aguilar 2005
 * (C) Copyright Kevlin Henney 2001
 *
 * Distributed under the Boost Software License, Version 1.0. (See
 * accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)  
 */

/////////////////////////////////////////////////////////////////////////////
// Updates:
// * 2006-07-17	- Added cast_(to<type>()) syntax
//				- Added #pragma once
//				- Added basic doxygen documentation
//
// * Before		- Sorry, didn't keep track of updates
/////////////////////////////////////////////////////////////////////////////

#ifndef CDIGGINS_ANY_HPP
#define CDIGGINS_ANY_HPP

#if defined(_MSC_VER) && (_MSC_VER >= 1020)
# pragma once
#endif

#include <stdexcept>
#include <typeinfo>
#include <algorithm>

/////////////////////////////////////////////////////////////////////////////
// Alignment handling
// Use boost when available. Otherwise, assume double is the type with the
// strictest alignment requirements
#ifdef ANY_DONT_USE_BOOST
#	define	ANY_MAX_ALIGN_TYPE double

#else	// ANY_DONT_USE_BOOST
#	include <boost/type_traits/alignment_of.hpp>
#	include <boost/type_traits/type_with_alignment.hpp>

#	define	ANY_MAX_ALIGN_TYPE boost::detail::max_align
#endif	// ANY_DONT_USE_BOOST
// Alignment handling
/////////////////////////////////////////////////////////////////////////////

/// cdiggins namespace.
/// There's been no need to change this so far. Besides, it gives credit to
/// Chris Diggins as the original author
namespace cdiggins
{
	/// Exception thrown when attempting to cast an \b any instance to a type
	/// different from what it currently holds.
	struct bad_any_cast
		: std::bad_cast 
	{
		bad_any_cast(
			  std::type_info const& src
			, std::type_info const& dest
		)
		  : from(src)
		  , to	(dest)
		{
		}

		virtual char const* what() const throw()
		{ 
			return "bad cast"; 
		}

		/// type_info of the type of the value stored by the any instance.
		std::type_info const& from;
		/// type_info of the type to which the stored value was attempted to be casted to.
		std::type_info const& to;
	};

/////////////////////////////////////////////////////////////////////////////
/// Contains constructs required for implementing any.
/// Not intended for public use.
namespace any_detail
{
	#ifndef ANY_BUFFER_SIZE
	enum
	{
		any_buffer_elements	= 1
	};
	#else	// defined(ANY_BUFFER_SIZE)
	enum
	{
		any_buffer_elements
			= (ANY_BUFFER_SIZE / sizeof(ANY_MAX_ALIGN_TYPE))			// Integer part
			+ ((ANY_BUFFER_SIZE % sizeof(ANY_MAX_ALIGN_TYPE)) ? 1 : 0)	// round upwards
	};
	#endif	// defined(ANY_BUFFER_SIZE)

	enum
	{
		any_buffer_size = sizeof(ANY_MAX_ALIGN_TYPE[any_buffer_elements])
	};

    /// Used to reset any values.
	struct empty { };
    
    /// Used to hold an object or a pointer to an object.
	union object_holder
	{
		/// Buffer for in-place storage of an object.
		char						buffer[any_buffer_size];
		/// Stores a pointer to the held object if it's stored in the heap.
		void*						pointer;

		/// Used to ensure proper alignment.
		ANY_MAX_ALIGN_TYPE			alignment_dummy_[any_buffer_elements];
	};	      

    /// Function pointer table.
    struct fxn_ptr_table
	{
		/// \brief Returns the type for which this function pointer table currently works.
		/// \returns The type for which this function pointer table currently works.
		std::type_info const&	(*type)();
		/// Destroys an in-place stored object.
		void (*destructor)		(object_holder&);
		/// Deletes an object held by pointer.
		void (*static_delete)	(object_holder&);
		/// Clones a stored object.
		void (*clone)			(object_holder&, const object_holder&);
		/// Copy assigns an object.
		void (*assign)			(object_holder&, const object_holder&);
		/// \brief Returns whether this type can be optimized or not.
		/// \returns Whether this type can be optimized or not.
		bool (*is_optimized)	();
		/// Swaps two values (of the same type!!).
		void (*swap)			(object_holder&, object_holder&);
    };

	// fxns implementation is coded with a nested template
	// to support compilers not having Partial Template Specialization support

	// static functions for small value-types 
	template<bool optimized>
	struct fxns
	{
		template<typename T>
		struct impl
		{
			static const std::type_info& type()
			{
				return typeid(T);
			}
			static void destructor(object_holder& x)
			{
				reinterpret_cast<T*>(x.buffer)->~T(); 
			}
			static void static_delete(object_holder& x)
			{
				destructor(x);
			}
			static void clone(
				  object_holder&		lvalue
				, object_holder const&	rvalue
			)
			{
				new(lvalue.buffer) T(
					*reinterpret_cast<T const*>(rvalue.buffer)
				);
			}
			static void assign(
				  object_holder&		lvalue
				, object_holder const&	rvalue)
			{
				*reinterpret_cast<T*>(lvalue.buffer) = *reinterpret_cast<T const*>(rvalue.buffer);
			}
			static bool is_optimized()
			{
				return true;
			}
			static void swap(object_holder& lhs, object_holder& rhs)
			{
				// Allow ADL [when available]
				using std::swap;
				swap(
					  *reinterpret_cast<T*>(lhs.buffer)
					, *reinterpret_cast<T*>(rhs.buffer)
				);
			}
		};
	};

    // static functions for big value-types (bigger than a void*)
	template<>
	struct fxns<false>
	{
		template<typename T>
		struct impl
		{
			static const std::type_info& type()
			{ 
				return typeid(T); 
			}
			static void destructor(object_holder& x)
			{
				reinterpret_cast<T*>(x.pointer)->~T(); 
			}
			static void static_delete(object_holder& x)
			{
				delete reinterpret_cast<T*>(x.pointer);
			}
			static void clone(
				  object_holder&		lvalue
				, object_holder const&	rvalue
			)
			{ 
				lvalue.pointer = new T(*reinterpret_cast<T const*>(rvalue.pointer)); 
			}
			static void assign(
				  object_holder&		lvalue
				, object_holder const&	rvalue
			)
			{ 
				*reinterpret_cast<T*>(lvalue.pointer) = *reinterpret_cast<T const*>(rvalue.pointer);
			}
			static bool is_optimized()
			{
				return false;
			}
			static void swap(object_holder& lhs, object_holder& rhs)
			{
				// Allow ADL [when available]
				using std::swap;
				swap(
					  *reinterpret_cast<T*>(lhs.pointer)
					, *reinterpret_cast<T*>(rhs.pointer)
				);
			}
		};
	};

	/// Metafunction to determine if type T can be optimized.
	template<typename T>
	struct optimize
	{
		enum
		{
			// Object's size must be equal or smaller than
			// available buffer space
			  T_size_		= sizeof(T)
			, base_size_	= sizeof(object_holder)
			, size_ok_		= (T_size_ <= base_size_)

			// Object's alignment requirements must be equal or
			// less strict than those our our buffer
#ifdef	ANY_DONT_USE_BOOST
			, align_ok_		= true
#else	// ANY_DONT_USE_BOOST
			, T_align_		= boost::alignment_of<T>::value
			, base_align_	= boost::alignment_of<object_holder>::value
			, align_ok_		= ((base_align_ % T_align_) == 0)
#endif	// ANY_DONT_USE_BOOST
			// Optimize only if both the above requirements are met
			, value			= size_ok_ && align_ok_
		};

		// Either the T doesn't fit, or it fits and is properly aligned
		// that's what max_align is supposed to guarantee isn't it?
#ifndef	ANY_DONT_USE_BOOST
		BOOST_STATIC_ASSERT((!size_ok_ || align_ok_));
#endif	// ANY_MAX_ALIGN_TYPE
	};

	/// Function table generator for type T.
    template<typename T>
    struct table
    {
		enum { optimized = optimize<T>::value };

	    static fxn_ptr_table* get()
	    {
		    static fxn_ptr_table static_table = {
				  &fxns<optimized>::template impl<T>::type
				, &fxns<optimized>::template impl<T>::destructor
				, &fxns<optimized>::template impl<T>::static_delete
				, &fxns<optimized>::template impl<T>::clone
				, &fxns<optimized>::template impl<T>::assign
				, &fxns<optimized>::template impl<T>::is_optimized
				, &fxns<optimized>::template impl<T>::swap
		    };
		    return &static_table;
	    }
    };

}	// namespace any_detail
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// Main class' implementation
/// Customization points:
/// \li \c ANY_DONT_USE_BOOST	Boost is used for maximum portability with alignment calculations, but it's use can be disabled defining this macro.
/// \li \c ANY_BUFFER_SIZE		Define this to be number of bytes of in-place storage space.
/// \li \c ANY_IMPLICIT_CASTING	Define this to enable implicit casting. Not recommended though.
///
/// \remarks The in-place buffer size could've been a non-type template
///          parameter. But that would've made this not-quite compatible with
///          using boost::any. So I'd rather just make it a configuration macro.
///
/// \todo * Improve exception safety on copy-assignment, if at all possible.
/// \todo * Move if( ... optimized ... ) logic, into function pointer table.
struct any
{
	/////////////////////////////////////////////////////////////////////////
	/// \name structors
	/// @{

	/// Constructs an \c any instance, holding a value.
	template <typename T>
	any(T const& x)
	{
		table = any_detail::table<T>::get();
		if( any_detail::optimize<T>::value )
		{
			new(held.buffer) T(x);
		}
		else
		{
			held.pointer = new T(x); 
		}
	}

	/// Constructs an empty any instance.
	any()
		: table(get_empty_table())
	{
	}

	/// Copy constructor.
	any(any const& x)
		: table(get_empty_table())
	{
		assign(x);
	}

	~any()
	{
		table->static_delete(held);
	}
	/// @}
	// structors
	/////////////////////////////////////////////////////////////////////////

	/////////////////////////////////////////////////////////////////////////
    /// \name Assignment
	/// @{
    template <typename T>
    any& assign(T const& x)
    {
		// are we copying between the same type?
		any_detail::fxn_ptr_table* x_table = any_detail::table<T>::get();
		if (table == x_table)
		{
			// I believe assignment would be better than
			// destryoing & contructing given that construction might fail
			// and leave us with a non existing object
			if( any_detail::optimize<T>::value )
			{
				*reinterpret_cast<T*>(held.buffer) = x;
			}
			else
			{
				*reinterpret_cast<T*>(held.pointer) = x;
			}
		}
		else
		{
			if( any_detail::optimize<T>::value )
			{
				// reset only if we're to store in-place
				reset();
				// create copy on-top of buffer
				new(held.buffer) T(x);
			}
			else
			{
				// copy first
				T* p = new T(x);
				// if successful, delete our current pointer, shouldn't throw
				table->static_delete(held);
				// keep new pointer
				held.pointer = p;
			}

			// update table pointer 
			table = x_table;
		}

		return *this;
	}

	any& assign(any const& x)
	{
		// are we copying between the same type?
		if (table == x.table)
		{
			// if so, we can avoid reallocation
			table->assign(held, x.held);
		}
		else
		{
			if( x.table->is_optimized() )
			{
				// reset only if we're to store in-place
				reset();
				x.table->clone(held, x.held);
			}
			else
			{
				// copy first
				any_detail::object_holder h;
				x.table->clone(h, x.held);
				// if successful, delete our current pointer, shouldn't throw
				table->static_delete(held);
				// keep new pointer
				held.pointer = h.pointer;
			}

			table = x.table;
		}

		return *this;
	}

    // assignment operator 
	template<typename T>
	any& operator=(T const& x)
	{
		return assign(x);
	}

	any& operator=(any const& x)
	{
		return assign(x);
	}
	
	bool operator==(any const& x) const
	{
		return type() == x.type() && memcmp(held.buffer, x.held.buffer, any_detail::any_buffer_size);
	}
	
	bool operator!=(any const& x) const
	{
		return !(operator==(x));
	}

	/// @}
	// Assignment
	/////////////////////////////////////////////////////////////////////////

    // utility functions
    any& swap(any& x)
	{
		bool opt_this	= table->is_optimized();
		bool opt_x		= x.table->is_optimized();

		if( !opt_this && !opt_x )
		{
			// both values are heap allocated
			// we can just swap pointers
			std::swap(table, x.table);
			std::swap(held.pointer, x.held.pointer);
		}
		else if( table == x.table )
		{
			// both values are of the same type
			// use std::swap with proper type information
			table->swap(held, x.held);
		}
		else //if( opt_this && opt_x )
		{
			// use fall-back technique to swap
			any temp(*this);
			*this	= x;
			x		= temp;
		}
//		else // exactly one is optimized
//		{
//			any& opt	= opt_this ? *this : x;
//			any& heap	= opt_this ? x : *this;
//
//			any_detail::fxn_ptr_table*	t	= heap.table;
//			void*						h	= heap.held.pointer;
//
//			heap = opt;
//
//			opt.reset();
//			opt.table			= t;
//			opt.held.pointer	= h;
//		}

		return *this;
    }

	/// Returns the type of the stored value/object.
	/// \returns The type of the stored value/object.
    std::type_info const& type() const
	{
		return table->type();
    }

	/////////////////////////////////////////////////////////////////////////
	/// \name Casting
	/// @{

	/// \brief Explicit cast method.
	/// <b>Can't be used with VC6.</b>
    template<typename T>
    T& cast()
	{
		// 'this' used to mimic any_cast's code
		if (this->type() != typeid(T))
		{
			throw bad_any_cast(this->type(), typeid(T));
		}

		if( any_detail::optimize<T>::value )
		{
			return *reinterpret_cast<T*>(this->held.buffer);
		}
		else
		{
			return *reinterpret_cast<T*>(this->held.pointer);
		}
    }

	/// Type holder for the VC6-specific cast_() method
	template<typename T>
	struct to { };

	/// \brief VC6-specific workaround casting method.
	/// VC6 doesn't support member template method calls with explicit
	/// template parameters. So this method takes an instance of a type holder
	/// instance, so that type to cast to can be determined.
    template<typename T>
    T& cast_(to<T>)
	{
		// 'this' used to mimic any_cast's code
		if (this->type() != typeid(T))
		{
			throw bad_any_cast(this->type(), typeid(T));
		}

		if( any_detail::optimize<T>::value )
		{
			return *reinterpret_cast<T*>(this->held.buffer);
		}
		else
		{
			return *reinterpret_cast<T*>(this->held.pointer);
		}
    }

	#ifdef ANY_IMPLICIT_CASTING
	/// Implicit casting operator.
	/// Disabled by default for compatibility with boost::any.
	template<typename T>
	operator T() const
	{
		return cast<T>();
	}
	#endif // ANY_IMPLICIT_CASTING

	/// @}
	// Casting
	/////////////////////////////////////////////////////////////////////////

	/// \returns The function pointer table to be used when nothing is stored.
	static any_detail::fxn_ptr_table* get_empty_table()
	{
		return any_detail::table<any_detail::empty>::get();
	}
    
	/// \returns True if there's no value/object stored.
	bool empty() const
	{
		return table == get_empty_table();
	}

	/// Get rid of the stored value/object and make this instance empty.
	void reset()
	{
		if( !empty() )
		{
			table->static_delete(held);
			table = get_empty_table();
		}
	}
	  	  
	/// Points to function pointer table that contains the methods for type-safely manipulating the stored object.
	any_detail::fxn_ptr_table*	table;
	/// Buffer to hold either a small object or a pointer to an object. 
	any_detail::object_holder	held;
};

	/////////////////////////////////////////////////////////////////////////
	// boost::any-like casting
	/////////////////////////////////////////////////////////////////////////
		/////////////////////////////////////////////////////////////////////
		// Pointer casts
		/////////////////////////////////////////////////////////////////////
		template<typename T>
		T* any_cast(any* this_)
		{
			if (this_->type() != typeid(T))
			{
				throw bad_any_cast(this_->type(), typeid(T));
			}

		#ifdef __BORLANDC__
		#   pragma warn -8008   // Condition is always true
		#   pragma warn -8066   // Unreachable code
		#endif
			if( any_detail::optimize<T>::value )
			{
				return reinterpret_cast<T*>(this_->held.buffer);
			}
			else
			{
				return reinterpret_cast<T*>(this_->held.pointer);
			}
		#ifdef __BORLANDC__
		#   pragma warn .8008   // Condition is always true
		#   pragma warn .8066   // Unreachable code
		#endif
		}

		template<typename T>
		T const* any_cast(any const* this_)
		{
			return any_cast<T>(const_cast<any*>(this_));
		}

		/////////////////////////////////////////////////////////////////////
		// Reference casts
		/////////////////////////////////////////////////////////////////////
		template<typename T>
		T& any_cast(any& this_)
		{
			return *any_cast<T>(&this_);
		}

		template<typename T>
		T const& any_cast(any const& this_)
		{
			return *any_cast<T>(const_cast<any*>(&this_));
		}
}

#endif // CDIGGINS_ANY_HPP
